"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.testCode = void 0;
const devopsCore_1 = require("@thatdevops/core/devopsCore");
const getProjectPkgs_1 = __importDefault(require("@thatdevops/core/pkgman/getProjectPkgs"));
const makeRunable_1 = require("@thatdevops/core/utils/makeRunable");
const spawnSilently_1 = __importDefault(require("@thatdevops/core/utils/spawnSilently"));
const toArray_1 = __importDefault(require("@thats/utils/array/toArray"));
const Logger_1 = __importDefault(require("@thats/utils/log/Logger"));
const fs_1 = __importDefault(require("fs"));
const chunk_1 = __importDefault(require("lodash/chunk"));
const flatten_1 = __importDefault(require("lodash/flatten"));
const path_1 = __importDefault(require("path"));
const parallel = 0;
exports.default = exports.testCode;
exports.testCode = (0, makeRunable_1.makeRunnable)((o) => __awaiter(void 0, void 0, void 0, function* () {
    const pkgList = yield (0, getProjectPkgs_1.default)();
    const { libraries, rootPath } = (0, devopsCore_1.getDevOpsCore)();
    Logger_1.default.shout('Testing Code');
    const toDos = pkgList
        .map(l => l.location)
        .filter((l) => libraries.some(p => o[p.label] && l.startsWith(path_1.default.join(rootPath, p.path))))
        .sort();
    if (!!parallel) {
        const chunks = (0, chunk_1.default)(toDos, parallel);
        for (const chunki of chunks) {
            yield Promise.all(chunki.map(testPackage));
        }
    }
    else {
        for (const toDo of toDos) {
            yield testPackage(toDo);
        }
    }
}));
function testPackage(dir) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const packageName = dir.split('/').reverse().slice(0, 2).reverse().join('/');
        const pkgJson = JSON.parse(fs_1.default.readFileSync(dir + '/package.json').toString());
        if ((_a = pkgJson === null || pkgJson === void 0 ? void 0 : pkgJson.devops) === null || _a === void 0 ? void 0 : _a.testIgnore) {
            Logger_1.default.info(' - SKIPPING ' + packageName);
            return;
        }
        const { nodeModulesPath } = (0, devopsCore_1.getDevOpsCore)();
        if (!fs_1.default.existsSync(dir + '/jest.config.js')) {
            Logger_1.default.info(' - SKIPPING ' + packageName + ' - no tests');
            return;
        }
        try {
            logStrArr(['-------------', ` ${packageName}`, '-------------']);
            const results = yield (0, spawnSilently_1.default)(`${nodeModulesPath}/.bin/jest`, [
                '--config', `${dir}/jest.config.js`,
                '--json',
                '--silent',
            ], undefined, true);
            if (results.length) {
                const resultsJson = JSON.parse((0, toArray_1.default)(results).join(''));
                logResults(dir, resultsJson);
            }
            else {
                logStrArr([`Total: 0`]);
            }
            Logger_1.default.info('');
        }
        catch (_b) { }
    });
}
function logResults(dir, r) {
    logStrArr([
        `Total: ${r.numTotalTests}`,
        `Passed: ${r.numPassedTests} (${Math.floor((r.numPassedTests / r.numTotalTests) * 100)}%)`,
        `Failed: ${r.numFailedTests} (${Math.floor((r.numFailedTests / r.numTotalTests) * 100)}%)`,
    ]);
    logStrArr([
        '-------------',
        ...(0, flatten_1.default)(r.testResults
            .filter(tr => tr.status === 'failed')
            .map(tr => [tr.name.split(dir)[1], tr.message])),
    ]);
}
function logStrArr(str) {
    Logger_1.default.info(str.join('\n'));
}
//# sourceMappingURL=testCode.js.map