"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tscCode = void 0;
const devopsCore_1 = require("@thatdevops/core/devopsCore");
const getProjectPkgs_1 = __importDefault(require("@thatdevops/core/pkgman/getProjectPkgs"));
const makeRunable_1 = require("@thatdevops/core/utils/makeRunable");
const spawn_1 = __importDefault(require("@thatdevops/core/utils/spawn"));
const Logger_1 = __importDefault(require("@thats/utils/log/Logger"));
const fs_1 = __importDefault(require("fs"));
const chunk_1 = __importDefault(require("lodash/chunk"));
const path_1 = __importDefault(require("path"));
exports.tscCode = (0, makeRunable_1.makeRunnable)((o) => __awaiter(void 0, void 0, void 0, function* () {
    const pkgList = yield (0, getProjectPkgs_1.default)();
    const { parallel = 0, libraries, rootPath } = (0, devopsCore_1.getDevOpsCore)();
    Logger_1.default.shout('TSCing Code');
    const toDos = pkgList
        .map(l => l.location)
        .filter((l) => libraries.some(p => o[p.label] && l.startsWith(path_1.default.join(rootPath, p.path))))
        .sort();
    if (!!parallel) {
        const chunks = (0, chunk_1.default)(toDos, parallel);
        for (const chunki of chunks) {
            yield Promise.all(chunki.map(tscPackage));
        }
    }
    else {
        for (const toDo of toDos) {
            yield tscPackage(toDo);
        }
    }
}));
exports.default = exports.tscCode;
function tscPackage(dir) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const packageName = dir.split('/').reverse().slice(0, 2).reverse().join('/');
        const pkgJson = JSON.parse(fs_1.default.readFileSync(dir + '/package.json').toString());
        if ((_a = pkgJson === null || pkgJson === void 0 ? void 0 : pkgJson.devops) === null || _a === void 0 ? void 0 : _a.tscIgnore) {
            Logger_1.default.info(' - SKIPPING ' + packageName);
            return;
        }
        if (!fs_1.default.existsSync(dir + '/tsconfig.json')) {
            Logger_1.default.info(' - SKIPPING ' + packageName + ' - not typescript');
            return;
        }
        const { nodeModulesPath } = (0, devopsCore_1.getDevOpsCore)();
        try {
            Logger_1.default.info(' - COMPILING ' + packageName);
            yield (0, spawn_1.default)(`${nodeModulesPath}/.bin/tsc`, ['-p', dir + '/tsconfig.json']);
        }
        catch (_b) { }
    });
}
//# sourceMappingURL=tscCode.js.map